#include "avr/io.h"

#include "i2c_hwdefs.h"

#define I2C_CONCAT(a, b)	a ## b
#define I2C_CONCAT_EXPANDED(a, b)   I2C_CONCAT(a, b)

#define I2C_PORT(name)	I2C_CONCAT(PORT, name)-__SFR_OFFSET
#define I2C_PIN(name)	I2C_CONCAT(PIN, name)-__SFR_OFFSET
#define I2C_DDR(name)	I2C_CONCAT(DDR, name)-__SFR_OFFSET


i2c_delay:
	;Zeitverzögerung 5µS (für 100kHz I2C-Takt)
	#if F_OSC>600000
	push	r16
	ldi	r16,F_OSC/600000
i2c_delay1:
	dec	r16
	brne	i2c_delay1
	pop	r16
	#endif
	ret


.global i2c_init
.func i2c_init
// void i2c_init(void);
i2c_init:
	;Ausgänge auf 1 setzen
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL bleibt erstmal hochohmig
	cbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;SDA bleibt erstmal hochohmig
	cbi	I2C_PORT(I2C_SCL_PORT),I2C_SCL_BIT	;bleibt auf Null, nur ddr geschaltet
	cbi	I2C_PORT(I2C_SDA_PORT),I2C_SDA_BIT	;bleibt auf Null, nur ddr geschaltet
	rjmp	i2c_delay
.endfunc


.global i2c_start
.func i2c_start
// void i2c_start(void);
i2c_start:
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch (für repeated Start)
	rcall	i2c_delay
	sbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;SDA runter
	rcall	i2c_delay
	sbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL runter
	rcall	i2c_delay
	ret
.endfunc


.global i2c_stop
.func i2c_stop
// void i2c_stop(void);
i2c_stop:
	sbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;SDA runter (wenn nicht schon)
	rcall	i2c_delay
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch
	rcall	i2c_delay
	cbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;SDA hoch
	rcall	i2c_delay
	ret
.endfunc


.global i2c_write
.func i2c_write
// uint8_t i2c_write(uint8_t);
	;Ausgabe r24 über i2c, return: ack=carry gelöscht
i2c_write:
	push	r17
	ldi	r17,8
i2c_wrbyt1:
	rol	r24
	brcc	i2c_wrbytnull
	cbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;eins ausgeben
	rjmp	i2c_wrbyteins
i2c_wrbytnull:
	sbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;null ausgeben
i2c_wrbyteins:
	rcall	i2c_delay
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch
	rcall	i2c_delay
i2c_wrbyte2:
	sbis	I2C_PIN(I2C_SCL_PORT),I2C_SCL_BIT	;ist SCL schon oben?
	rjmp	i2c_wrbyte2
	sbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL runter
	rcall	i2c_delay
	dec	r17
	brne	i2c_wrbyt1
	cbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;SDA hochohmig
	rcall	i2c_delay
	;Neunten Impuls ausgeben
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch
	rcall	i2c_delay
i2c_wrbyte3:
	sbis	I2C_PIN(I2C_SCL_PORT),I2C_SCL_BIT	;ist SCL schon oben?
	rjmp	i2c_wrbyte3
	clr	r24					;Ack-Bit in r24 übernehmen
	sbic	I2C_PIN(I2C_SDA_PORT),I2C_SDA_BIT
	ser	r24
	sbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL runter
	rcall	i2c_delay
	pop	r17
	ret
.endfunc


.global i2c_read
.func i2c_read
// uint8_t i2c_read(void);
	;Eingabe über i2c, return: R24
i2c_read:
	push	r17
	ldi	r17,8
i2c_rdbyt1:
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch
	rcall	i2c_delay
i2c_rdbyte2:
	sbis	I2C_PIN(I2C_SCL_PORT),I2C_SCL_BIT	;ist SCL schon oben?
	rjmp	i2c_rdbyte2
	clc				;SDA in carry kopieren
	sbic	I2C_PIN(I2C_SDA_PORT),I2C_SDA_BIT
	sec
	rol	r24			;und einschieben
	sbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL runter
	rcall	i2c_delay
	dec	r17
	brne	i2c_rdbyt1
	brcs	i2c_rdbyt_nack		;Carry: kein ack!
	sbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;Ack ausgeben
i2c_rdbyt_nack:
	rcall	i2c_delay
	cbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL hoch
	rcall	i2c_delay
i2c_rdbyte3:
	sbis	I2C_PIN(I2C_SCL_PORT),I2C_SCL_BIT	;ist SCL schon oben?
	rjmp	i2c_rdbyte3
	sbi	I2C_DDR(I2C_SCL_PORT),I2C_SCL_BIT	;SCL runter
	rcall	i2c_delay
	cbi	I2C_DDR(I2C_SDA_PORT),I2C_SDA_BIT	;Ack beenden
	pop	r17
	ret
.endfunc
