#include <avr/io.h>

#include "tnarx_hwdefs.h"

#define LCD_CONCAT(a, b)	a ## b
#define LCD_CONCAT_EXPANDED(a, b)   LCD_CONCAT(a, b)

#define LCD_PORT(name)	LCD_CONCAT(PORT, name)-__SFR_OFFSET
#define LCD_DDR(name)	LCD_CONCAT(DDR, name)-__SFR_OFFSET

lcd_obyte:
	push	r17
	ldi	r17,8
lcd_obyte_loop:
	lsl	r16
	brlo	lcd_obyte_1	;Carry gesetzt
	cbi	LCD_PORT(lcd_dat_port),lcd_dat_bit
	rjmp	lcd_obyte_common
lcd_obyte_1:
	sbi	LCD_PORT(lcd_dat_port),lcd_dat_bit
lcd_obyte_common:
	nop
	sbi	LCD_PORT(lcd_clk_port),lcd_clk_bit
	nop
	cbi	LCD_PORT(lcd_clk_port),lcd_clk_bit
	dec	r17
	brne	lcd_obyte_loop
	pop	r17
	ret

	;==========================================
.global lcd_init
.func lcd_init
lcd_init:
	sbi	LCD_DDR(lcd_clk_port),lcd_clk_bit
	sbi	LCD_DDR(lcd_dat_port),lcd_dat_bit
	sbi	LCD_DDR(lcd_ena_port),lcd_ena_bit
	ret
.endfunc

.global lcd_out
.func lcd_out
lcd_out:
	push	zh
	push	zl
	push	r17
	push	r16
	mov	zl,r24
	mov	zh,r25

	sbi	LCD_PORT(lcd_ena_port),lcd_ena_bit
	ldi	r17,6
lcd_out1:
	ld	r16,z+
	rcall	lcd_obyte
	dec	r17
	brne	lcd_out1
	ldi	r16,4
	rcall	lcd_obyte
	cbi	LCD_PORT(lcd_ena_port),lcd_ena_bit
	
	sbi	LCD_PORT(lcd_ena_port),lcd_ena_bit
	ldi	r17,6
lcd_out2:
	ld	r16,z+
	rcall	lcd_obyte
	dec	r17
	brne	lcd_out2
	ldi	r16,5
	rcall	lcd_obyte
	cbi	LCD_PORT(lcd_ena_port),lcd_ena_bit
	cbi	LCD_PORT(lcd_dat_port),lcd_dat_bit
	pop	r16
	pop	r17
	pop	zl
	pop	zh
	ret
.endfunc



.global Segmenttabelle
Segmenttabelle:
	;	  76543210   76543210
	.byte	0b10110111,0b10010000	;0,1
	.byte	0b00111110,0b10111100	;2,3
	.byte	0b10011001,0b10101101	;4,5
	.byte	0b10101111,0b10010100	;6,7
	.byte	0b10111111,0b10111101	;8,9
	.byte	0b10011111,0b10101011	;A,b
	.byte	0b00100111,0b10111010	;C,d
	.byte	0b00101111,0b00001111	;E,F


.global lword
lword:
	push	zh
	push	zl
	push	yh
	push	yl
	push	r21
	push	r20
	push	r19
	push	r18
	push	r17
	push	r16

	mov	yl,r22	;Cursorposition
	mov	yh,r23

	clr	r22	;Vorzeichen
	sbrs	r25,7	;Minus?
	rjmp	div
	;Vorzeichen umdrehen
	ldi	r22,0x8	;Minus
	clr	r16
	clr	r17
	sub	r16,r24
	sbc	r17,r25
	mov	r24,r16
	mov	r25,r17

	;versuche RR24:25 durch 10 zu teilen
	;Rest in R16
	;Quotient in RR24:25
div:
	clr	r18	;divisor RR18:19=10<<12
	ldi	r19,10<<4
	ldi	r17,13
	clr	r20	;Quotient
	clr	r21
div1:
	lsl	r20	;Quotient linksschieben
	rol	r21
	ori	r20,1
	sub	r24,r18
	sbc	r25,r19
	brcc	div2
	andi	r20,~1	;Rollback erforderlich
	add	r24,r18
	adc	r25,r19
div2:
	lsr	r19	;Divisor rechtsschieben
	ror	r18
	dec	r17
	brne	div1
	;R24:25 hat jetzt den Rest
	ldi 	zh, hi8(Segmenttabelle)
	ldi 	zl, lo8(Segmenttabelle)
	add	zl,r24
	adc	zh,r25
	lpm 	r16,z
	st	y+,r16
	mov	r24,r20
	mov	r25,r21
	or	r20,r21	;nächste Runde?
	brne	div
	st	y+,r22	;Vorzeichen
	pop	r16
	pop	r17
	pop	r18
	pop	r19
	pop	r20
	pop	r21
	pop	yl
	pop	yh
	pop	zl
	pop	zh
	ret


